from django.test import TestCase
from aws_message.message import (
    extract_inner_message, validate_message_signature)

mbody = {
    u'SignatureVersion': u'1',
    u'Timestamp': u'2018-08-22T04:00:56.843Z',
    u'Signature': u'...',
    u'SigningCertURL': (
        u'https://sss.qq-qqqq-1.aaaaaaaaa.com/SimpleNotificationService-'
        u'00000000000000000000000000000000.pem'),
    u'MessageId': u'11111111-0000-1111-2222-555555555555',
    u'Message': (
        u'{"EventID":"00000000-1111-2222-3333-444444444444","Href":'
        u'"...","EventDate":"2018-08-21T21:00:56.832068-07:00","Previous":'
        u'{"CurrentEnrollment":19,"Status":"open"},"Current":'
        u'{"CurrentEnrollment":20,"Status":"closed"}}'),
    u'UnsubscribeURL': (
        u'https://sss.qq-qqqq-1.aaaaaaaaa.com/?Action=Unsubscribe&'
        u'SubscriptionArn=arn:aaa:sss:qq-qqqq-1:...:...:...'),
    u'Type': u'Notification',
    u'TopicArn': u'arn:aaa:sas:qq-qqqq-1:...:...:...',
    u'Subject': u'UW Event',
}


class TestMockProcessor(TestCase):

    def test_extract_inner_message(self):
        message = extract_inner_message(mbody)
        self.assertTrue("EventID" in message)
        self.assertTrue("Href" in message)
        self.assertTrue("EventDate" in message)
        self.assertTrue("Previous" in message)
        self.assertTrue("Current" in message)

    def test_validate_message_signature(self):
        with self.settings(AWS_CA_BUNDLE="/data/"):
            try:
                validate_message_signature(mbody)
            except AttributeError as ex:
                # python 3.6: module 'string' has no attribute 'lower'
                # line 44: Signature(sig_conf).validate(_signText(mbody)
                pass
            except Exception as ex:
                pass



"""
Validate and decode AWS SNS/SQS message
"""
from django.conf import settings
import urllib3
import json
import re
from base64 import b64decode
from aws_message.crypto import Signature, CryptoException


class SNSException(Exception):
    pass


b64encoded = re.compile(r'^[a-zA-Z0-9]+[=]{0,2}$')


def extract_inner_message(mbody):
    message = mbody.get('Message')

    if isinstance(message, dict):
        return message

    if b64encoded.match(message):
        message = b64decode(message)

    return json.loads(message)


def validate_message_signature(mbody):
    """
    raises CryptoException, SNSException
    """
    t = mbody['SignatureVersion']
    if t != '1':
        raise SNSException('Unknown SNS Signature Version: ' + t)

    sig_conf = {
        'cert': {
            'type': 'url',
            'reference': mbody['SigningCertURL']
        }
    }

    try:
        Signature(sig_conf).validate(_signText(mbody),
                                     b64decode(mbody['Signature']))
    except Exception as err:
        raise SNSException('validate_message_body: {} ({})'.format(err, mbody))


def _signText(mbody):
    to_sign = ''
    # see: ("http://docs.amazonwebservices.com/sns/latest/"
    #       "gsg/SendMessageToHttp.verify.signature.html")

    to_sign = _sigElement('Message', mbody) + _sigElement('MessageId', mbody)

    if mbody['Type'] == 'Notification':
        if 'Subject' in mbody:
            to_sign += _sigElement('Subject', mbody)
        to_sign += _sigElement('Timestamp', mbody)

    elif mbody['Type'] == 'SubscriptionConfirmation':
        to_sign += _sigElement('SubscribeURL', mbody)
        to_sign += _sigElement('Timestamp', mbody)
        to_sign += _sigElement('Token', mbody)

    to_sign += _sigElement('TopicArn', mbody)
    to_sign += _sigElement('Type', mbody)

    return to_sign.encode('utf-8')


def _sigElement(el, mbody):
    return '{}\n{}\n'.format(el, mbody[el])


def subscribe(mbody):
    try:
        http = urllib3.PoolManager(
            cert_reqs='CERT_REQUIRED',
            ca_certs=settings.AWS_CA_BUNDLE
        )
        r = http.request('GET', mbody['SubscribeURL'])
        if r.status != 200:
            raise SNSException(
                'Subscribe to {} failure: status: {}'.format(
                    mbody['TopicArn'], r.status))
    except urllib3.exceptions.HTTPError as err:
        raise SNSException('Subscribe to {} failure: {}'.format(
            mbody['TopicArn'], err))



import logging
from abc import ABC, abstractmethod
from django.conf import settings


class ProcessorException(Exception):
    pass


class InnerMessageProcessor(ABC):
    def __init__(self, logger, queue_settings_name, is_encrypted=False):
        self.logger = logger
        self.settings = settings.AWS_SQS.get(queue_settings_name, {})
        self.is_encrypted = is_encrypted

    def validate_inner_message(self, message):
        """
        Override in the sub-class if the inner message requires validation.
        """
        pass

    def validate_inner_message_signature(self, message):
        """
        Override in the sub-class if the inner message signature requires
        validation.
        """
        pass

    def decrypt_inner_message(self, message):
        """
        Override in the sub-class if the inner message is encrypted
        """
        return message

    def get_payload_settings(self):
        return self.settings.get('PAYLOAD_SETTINGS', {})

    def get_queue_settings(self):
        return self.settings

    def process(self, message):
        """
        :param message: the inner message json data
        """
        self.validate_inner_message(message)

        if self.get_payload_settings().get('VALIDATE_MSG_SIGNATURE', False):
            self.validate_inner_message_signature(message)

        if self.is_encrypted:
            message = self.decrypt_inner_message(message)

        self.process_inner_message(message)

    @abstractmethod
    def process_inner_message(self, json_data):
        """
        A sub-class must define this method
        :raises ProcessorException: any error unable to handle
        """
        pass
